///////////////////////////////////////////////////////////////////////////////
//
// COMMON ATTRIBUTES
//
///////////////////////////////////////////////////////////////////////////////
attribute mediump vec4 a_boneIndices;
attribute mediump vec4 a_boneWeights;

///////////////////////////////////////////////////////////////////////////////
//
// SHADER TECHNIQUES VALIDATION
//
///////////////////////////////////////////////////////////////////////////////
#if defined(SHADER_FLAG_SKINNING_MATRIX4) || defined(SHADER_FLAG_SKINNING_TRS)
#define USING_SUPPORTED_SHADER_SKINNING_TECHNIQUE
#endif

///////////////////////////////////////////////////////////////////////////////
//
// BONES_PER_VERTEX
//
///////////////////////////////////////////////////////////////////////////////
#if defined(SHADER_FLAG_SKINNING_BONES_PER_VERTEX_1)
#define BONES_PER_VERTEX 1
#define SampleBoneWeight(index_) (a_boneWeights[index_])

///////////////////////////////////////////////////////////////////////////////
#elif defined(SHADER_FLAG_SKINNING_BONES_PER_VERTEX_2)
#define BONES_PER_VERTEX 2
#define SampleBoneWeight(index_) (a_boneWeights[index_])

///////////////////////////////////////////////////////////////////////////////
#elif defined(SHADER_FLAG_SKINNING_BONES_PER_VERTEX_3)
#define BONES_PER_VERTEX 3
#define SampleBoneWeight(index_) (a_boneWeights[index_])

///////////////////////////////////////////////////////////////////////////////
#elif defined(SHADER_FLAG_SKINNING_BONES_PER_VERTEX_4)
#define BONES_PER_VERTEX 4

uniform int u_numBoneInfluencesPerVertex;

float GetMaskedBoneWeight(float boneWeight, int index) {
	// OpenGL expands attributes with y=0, z=0 and w=1, so we need to selectively mask out
	// the default w expansion since we might have bone weight vertex buffers with fewer than four elements.
	// This is only an issue when we are forcing four bones per vertex since it's the only time we might be
	// using the w component without a value for it in the vertex buffer.

	// Since we are already setting up the u_numBoneInfluencesPerVertex uniform for the dyn branching version 
	// of the shader, we can use it to obtain a mask value for the weight.
	// Alternatively we could introduce a new uniform with the weight mask already calculated.

	float boneWeightMask = step(1.0, float(u_numBoneInfluencesPerVertex) / float(index));
	return boneWeight * boneWeightMask;
}

#define SampleBoneWeight(index_) GetMaskedBoneWeight(a_boneWeights[index_], index_)

///////////////////////////////////////////////////////////////////////////////
#elif defined(USING_SUPPORTED_SHADER_SKINNING_TECHNIQUE)

uniform int u_numBoneInfluencesPerVertex;

#define BONES_PER_VERTEX u_numBoneInfluencesPerVertex
#define SampleBoneWeight(index_) (a_boneWeights[index_])

///////////////////////////////////////////////////////////////////////////////
#else
#define BONES_PER_VERTEX 0

#endif

#define SampleBoneIndex(index_) (int(a_boneIndices[index_]))

///////////////////////////////////////////////////////////////////////////////
//
// TRS SKINNING
//
///////////////////////////////////////////////////////////////////////////////
#if defined(SHADER_FLAG_SKINNING_TRS)

struct TransformTRS {
	highp vec4 translationScale;
	highp vec4 rotation;
};

#define MAX_BONES_PER_MESH 60
#define NUM_VEC4_IN_TRS_STRUCT 2

// cannot use array of structs with glGetAttributeLocation in gles
uniform highp vec4 u_boneTransforms[MAX_BONES_PER_MESH * NUM_VEC4_IN_TRS_STRUCT];

TransformTRS GetBoneTransform(int boneIndex) {
	int transformIndex = int(boneIndex) * NUM_VEC4_IN_TRS_STRUCT;
	TransformTRS res;
	res.translationScale = u_boneTransforms[transformIndex];
	res.rotation = u_boneTransforms[transformIndex + 1];
	return res;
}

vec3 MulTransformVector(TransformTRS trans, highp vec3 vertex) {
	vec3 t = 2.0 * cross(trans.rotation.yzw , vertex);
	vec3 res = vertex + trans.rotation.x * t + cross(trans.rotation.yzw, t);
	return (res * trans.translationScale.w + trans.translationScale.xyz);
}

vec3 CalculateSkinnedPositionContribution(highp vec3 position, int boneIndex, float boneWeight) {
	TransformTRS boneTransform = GetBoneTransform(boneIndex);
	return MulTransformVector(boneTransform, position) * boneWeight;
}

vec4 CalculateSkinnedPosition(highp vec4 position) {
	highp vec3 skinnedPosition = vec3(0.0, 0.0, 0.0);
	
	{
		skinnedPosition += CalculateSkinnedPositionContribution(position.xyz, SampleBoneIndex(0), SampleBoneWeight(0));
	}
	
	if (2 <= BONES_PER_VERTEX) {
		skinnedPosition += CalculateSkinnedPositionContribution(position.xyz, SampleBoneIndex(1), SampleBoneWeight(1));
	}
	
	if (3 <= BONES_PER_VERTEX) {
		skinnedPosition += CalculateSkinnedPositionContribution(position.xyz, SampleBoneIndex(2), SampleBoneWeight(2));
	}
	
	if (4 <= BONES_PER_VERTEX) {
		skinnedPosition += CalculateSkinnedPositionContribution(position.xyz, SampleBoneIndex(3), SampleBoneWeight(3));
	}

	return vec4(skinnedPosition, 1.0);
}

///////////////////////////////////////////////////////////////////////////////
//
// MATRIX4 SKINNING
//
///////////////////////////////////////////////////////////////////////////////
#elif defined(SHADER_FLAG_SKINNING_MATRIX4)

#define MAX_BONES_PER_MESH 30
uniform highp mat4 u_boneMatrices[MAX_BONES_PER_MESH];


vec4 CalculateSkinnedPositionContribution(highp vec4 position, int boneIndex, float boneWeight) {
	return u_boneMatrices[boneIndex] * position * boneWeight;
}

 vec4 CalculateSkinnedPosition(highp vec4 position) {
	highp vec4 skinnedPosition = vec4(0.0, 0.0, 0.0, 0.0);
	
	{
		skinnedPosition += CalculateSkinnedPositionContribution(position, SampleBoneIndex(0), SampleBoneWeight(0));
	}
	
	if (2 <= BONES_PER_VERTEX) {
		skinnedPosition += CalculateSkinnedPositionContribution(position, SampleBoneIndex(1), SampleBoneWeight(1));
	}
	
	if (3 <= BONES_PER_VERTEX) {
		skinnedPosition += CalculateSkinnedPositionContribution(position, SampleBoneIndex(2), SampleBoneWeight(2));
	}
	
	if (4 <= BONES_PER_VERTEX) {
		skinnedPosition += CalculateSkinnedPositionContribution(position, SampleBoneIndex(3), SampleBoneWeight(3));
	}
	
	return skinnedPosition;
}

///////////////////////////////////////////////////////////////////////////////
//
// NO SKINNING
//
///////////////////////////////////////////////////////////////////////////////
#else

vec4 CalculateSkinnedPosition(highp vec4 position) {
	return position;
}

#endif